#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import cmind as cm
import os
import json
import re
import shutil


def preprocess(i):

    os_info = i['os_info']
    env = i['env']

    if env["CM_DOCKER_OS"] not in ["ubuntu", "rhel", "arch"]:
        return {
            'return': 1, 'error': f"Specified docker OS: {env['CM_DOCKER_OS']}. Currently only ubuntu, rhel and arch are supported in CM docker"}

    path = i['run_script_input']['path']

    with open(os.path.join(path, "dockerinfo.json")) as f:
        config = json.load(f)

    build_args = []
    build_args_default = {}
    input_args = []
    copy_files = []

    if env.get('CM_DOCKER_RUN_SCRIPT_TAGS', '') != '':
        script_tags = env['CM_DOCKER_RUN_SCRIPT_TAGS']
        found_scripts = cm.access(
            {'action': 'search', 'automation': 'script', 'tags': script_tags})
        scripts_list = found_scripts['list']

        if not scripts_list:
            return {'return': 1,
                    'error': 'No CM script found for tags ' + script_tags}

        if len(scripts_list) > 1:
            return {
                'return': 1, 'error': 'More than one scripts found for tags ' + script_tags}

        script = scripts_list[0]
        input_mapping = script.meta.get('input_mapping', {})
        default_env = script.meta.get('default_env', {})

        for input_, env_ in input_mapping.items():
            if input_ == "docker":
                continue
            arg = env_
            if env_ in default_env:  # other inputs to be done later
                arg = arg + "=" + str(default_env[env_])
                # build_args.append(arg)
                # input_args.append("--"+input_+"="+"$"+env_)

    if "CM_DOCKER_OS_VERSION" not in env:
        env["CM_DOCKER_OS_VERSION"] = "20.04"

    docker_image_base = get_value(env, config, 'FROM', 'CM_DOCKER_IMAGE_BASE')
    if not docker_image_base:
        return {
            'return': 1, 'error': f"Version \"{env['CM_DOCKER_OS_VERSION']}\" is not supported yet for \"{env['CM_DOCKER_OS']}\" "}

    # Handle cm_mlops Repository
    if env.get("CM_REPO_PATH", "") != "":
        use_copy_repo = True
        cm_repo_path = os.path.abspath(env["CM_REPO_PATH"])

        if not os.path.exists(cm_repo_path):
            return {
                'return': 1, 'error': f"Specified CM_REPO_PATH does not exist: {cm_repo_path}"}

        cmr_yml_path = os.path.join(cm_repo_path, "cmr.yaml")
        if not os.path.isfile(cmr_yml_path):
            return {
                'return': 1, 'error': f"cmr.yaml not found in CM_REPO_PATH: {cm_repo_path}"}

        # Define the build context directory (where the Dockerfile will be)
        build_context_dir = os.path.dirname(
            env.get(
                'CM_DOCKERFILE_WITH_PATH',
                os.path.join(
                    os.getcwd(),
                    "Dockerfile")))
        os.makedirs(build_context_dir, exist_ok=True)

        # Create cm_repo directory relative to the build context
        repo_build_context_path = os.path.join(build_context_dir, "cm_repo")

        # Remove existing directory if it exists
        if os.path.exists(repo_build_context_path):
            shutil.rmtree(repo_build_context_path)

        try:
            print(
                f"Copying repository from {cm_repo_path} to {repo_build_context_path}")
            shutil.copytree(cm_repo_path, repo_build_context_path)
        except Exception as e:
            return {
                'return': 1, 'error': f"Failed to copy repository to build context: {str(e)}"}

        if not os.path.isdir(repo_build_context_path):
            return {
                'return': 1, 'error': f"Repository was not successfully copied to {repo_build_context_path}"}

        # (Optional) Verify the copy
        if not os.path.isdir(repo_build_context_path):
            return {
                'return': 1, 'error': f"cm_repo was not successfully copied to the build context at {repo_build_context_path}"}
        else:
            print(
                f"cm_repo is present in the build context at {repo_build_context_path}")

        relative_repo_path = os.path.relpath(
            repo_build_context_path, build_context_dir)
    else:
        # CM_REPO_PATH is not set; use cm pull repo as before
        use_copy_repo = False

        if env.get("CM_MLOPS_REPO", "") != "":
            cm_mlops_repo = env["CM_MLOPS_REPO"]
            # the below pattern matches both the HTTPS and SSH git link formats
            git_link_pattern = r'^(https?://github\.com/([^/]+)/([^/]+)(?:\.git)?|git@github\.com:([^/]+)/([^/]+)(?:\.git)?)$'
            if match := re.match(git_link_pattern, cm_mlops_repo):
                if match.group(2) and match.group(3):
                    repo_owner = match.group(2)
                    repo_name = match.group(3)
                elif match.group(4) and match.group(5):
                    repo_owner = match.group(4)
                    repo_name = match.group(5)
                cm_mlops_repo = f"{repo_owner}@{repo_name}"
                print(
                    f"Converted repo format from {env['CM_MLOPS_REPO']} to {cm_mlops_repo}")
        else:
            cm_mlops_repo = "mlcommons@mlperf-automations"

    cm_mlops_repo_branch_string = f" --branch={env['CM_MLOPS_REPO_BRANCH']}"

    if env.get('CM_DOCKERFILE_WITH_PATH', '') == '':
        env['CM_DOCKERFILE_WITH_PATH'] = os.path.join(
            os.getcwd(), "Dockerfile")

    dockerfile_with_path = env['CM_DOCKERFILE_WITH_PATH']
    dockerfile_dir = os.path.dirname(dockerfile_with_path)

    extra_dir = os.path.dirname(dockerfile_with_path)

    if extra_dir != '':
        os.makedirs(extra_dir, exist_ok=True)

    f = open(dockerfile_with_path, "w")
    EOL = env['CM_DOCKER_IMAGE_EOL']
    f.write('FROM ' + docker_image_base + EOL)

    # Maintainers
    f.write(EOL)
    f.write(
        '# Automatically generated by the CM workflow automation meta-framework' +
        EOL)
    f.write('# https://github.com/mlcommons/ck' + EOL)
    f.write(EOL)

    f.write('LABEL github=""' + EOL)
    f.write('LABEL maintainer=""' + EOL)
    f.write('LABEL license=""' + EOL)

    f.write(EOL)

    image_label = get_value(env, config, 'LABEL', 'CM_DOCKER_IMAGE_LABEL')
    if image_label:
        f.write('LABEL ' + image_label + EOL)
        f.write(EOL)

    shell = get_value(env, config, 'SHELL', 'CM_DOCKER_IMAGE_SHELL')
    if shell:
        # f.write('SHELL ' + shell + EOL)
        f.write(EOL)

    for arg in config['ARGS_DEFAULT']:
        arg_value = config['ARGS_DEFAULT'][arg]
        f.write('ARG ' + f"{arg}={arg_value}" + EOL)

    for arg in config['ARGS']:
        f.write('ARG ' + arg + EOL)

    for build_arg in build_args:
        f.write('ARG ' + build_arg + EOL)

    for build_arg in sorted(build_args_default):
        v = build_args_default[build_arg]
        f.write('ARG ' + build_arg + '="' + str(v) + '"' + EOL)

    f.write(EOL)
    copy_cmds = []
    if 'CM_DOCKER_COPY_FILES' in env:
        for copy_file in env['CM_DOCKER_COPY_FILES']:
            copy_split = copy_file.split(":")
            if len(copy_split) != 2:
                return {
                    'return': 1, 'error': 'Invalid docker copy input {} given'.format(copy_file)}
            filename = os.path.basename(copy_split[0])
            if not os.path.exists(os.path.join(dockerfile_dir, filename)):
                shutil.copytree(
                    copy_split[0], os.path.join(
                        dockerfile_dir, filename))
            f.write('COPY ' + filename + " " + copy_split[1] + EOL)

    f.write(
        EOL +
        '# Notes: https://runnable.com/blog/9-common-dockerfile-mistakes' +
        EOL +
        '# Install system dependencies' +
        EOL)
    f.write(
        'RUN ' +
        get_value(
            env,
            config,
            'package-manager-update-cmd',
            'CM_PACKAGE_MANAGER_UPDATE_CMD') +
        EOL)
    f.write('RUN ' + get_value(env, config, 'package-manager-get-cmd') + " " + " ".join(get_value(env, config,
                                                                                                  'packages')) + EOL)

    if env.get('CM_DOCKER_EXTRA_SYS_DEPS', '') != '':
        f.write('RUN ' + env['CM_DOCKER_EXTRA_SYS_DEPS'] + EOL)

    if env['CM_DOCKER_OS'] == "ubuntu":
        if int(env['CM_DOCKER_OS_VERSION'].split('.')[0]) >= 23:
            if "--break-system-packages" not in env.get(
                    'CM_DOCKER_PIP_INSTALL_EXTRA_FLAGS', ''):
                env['CM_DOCKER_PIP_INSTALL_EXTRA_FLAGS'] = " --break-system-packages"
    pip_extra_flags = env.get('CM_DOCKER_PIP_INSTALL_EXTRA_FLAGS', '')

    f.write(EOL + '# Setup docker environment' + EOL)

    entry_point = get_value(
        env,
        config,
        'ENTRYPOINT',
        'CM_DOCKER_IMAGE_ENTRYPOINT')
    if entry_point:
        f.write('ENTRYPOINT ' + entry_point + EOL)

    for key, value in config['ENV'].items():
        f.write('ENV ' + key + "=\"" + value + "\"" + EOL)
    for cmd in config['RUN_CMDS']:
        f.write('RUN ' + cmd + EOL)

    f.write(EOL + '# Setup docker user' + EOL)
    docker_user = get_value(env, config, 'USER', 'CM_DOCKER_USER')
    docker_group = get_value(env, config, 'GROUP', 'CM_DOCKER_GROUP')

    if env.get('CM_CONTAINER_TOOL', '') == 'podman' and env.get(
            'CM_DOCKER_USE_DEFAULT_USER', '') == '':
        env['CM_DOCKER_USE_DEFAULT_USER'] = 'yes'

    if docker_user and str(env.get('CM_DOCKER_USE_DEFAULT_USER', '')).lower() not in [
            "yes", "1", "true"]:

        f.write('RUN groupadd -g $GID -o ' + docker_group + EOL)

        DOCKER_USER_ID = "-m -u $UID "
        DOCKER_GROUP = "-g $GID -o"

        user_shell = json.loads(shell)
        f.write('RUN useradd ' + DOCKER_USER_ID + DOCKER_GROUP + ' --create-home --shell ' + user_shell[0] + ' '
                + docker_user + EOL)
        f.write(
            'RUN echo "' +
            docker_user +
            ' ALL=(ALL) NOPASSWD: ALL" >> /etc/sudoers' +
            EOL)
        f.write('USER ' + docker_user + ":" + docker_group + EOL)
        f.write('ENV HOME=/home/cmuser' + EOL)

    else:
        f.write('ENV HOME=/root' + EOL)

    dockerfile_env = env.get('CM_DOCKERFILE_ENV', {})
    dockerfile_env_input_string = ""
    for docker_env_key in dockerfile_env:
        dockerfile_env_input_string = dockerfile_env_input_string + " --env." + \
            docker_env_key + "=" + str(dockerfile_env[docker_env_key])

    workdir = get_value(env, config, 'WORKDIR', 'CM_DOCKER_WORKDIR')
    if workdir and ("/home/cmuser" not in workdir or str(env.get('CM_DOCKER_USE_DEFAULT_USER', '')).lower() not in [
            "yes", "1", "true"]):
        f.write('WORKDIR ' + workdir + EOL)

    f.write(EOL + '# Install python packages' + EOL)
    python = get_value(env, config, 'PYTHON', 'CM_DOCKERFILE_PYTHON')

    docker_use_virtual_python = env.get('CM_DOCKER_USE_VIRTUAL_PYTHON', "yes")
    if str(docker_use_virtual_python).lower() not in ["no", "0", "false"]:
        f.write('RUN {} -m venv $HOME/venv/cm'.format(python) + " " + EOL)
        f.write('ENV PATH="$HOME/venv/cm/bin:$PATH"' + EOL)
    # f.write('RUN . /opt/venv/cm/bin/activate' + EOL)

    f.write(
        'RUN {} -m pip install '.format(python) +
        " ".join(
            get_value(
                env,
                config,
                'python-packages')) +
        ' ' +
        pip_extra_flags +
        ' ' +
        EOL)

    f.write(EOL + '# Download CM repo for scripts' + EOL)

    if use_copy_repo:
        docker_repo_dest = "$HOME/CM/repos/mlcommons@mlperf-automations"
        f.write(
            f'COPY --chown=cmuser:cm {relative_repo_path} {docker_repo_dest}' +
            EOL)

        f.write(EOL + '# Register CM repository' + EOL)
        f.write('RUN cm pull repo --url={} --quiet'.format(docker_repo_dest) + EOL)
        f.write(EOL)

    else:
        # Use cm pull repo as before
        x = env.get('CM_DOCKER_ADD_FLAG_TO_CM_MLOPS_REPO', '')
        if x != '':
            x = ' ' + x

        f.write(
            'RUN cm pull repo ' +
            cm_mlops_repo +
            cm_mlops_repo_branch_string +
            x +
            EOL)

    # Check extra repositories
    x = env.get('CM_DOCKER_EXTRA_CM_REPOS', '')
    if x != '':
        for y in x.split(','):
            f.write('RUN ' + y + EOL)

    if str(env.get('CM_DOCKER_SKIP_CM_SYS_UPGRADE', False)
           ).lower() not in ["true", "1", "yes"]:
        f.write(EOL + '# Install all system dependencies' + EOL)
        f.write('RUN cm run script --tags=get,sys-utils-cm --quiet' + EOL)

    if 'CM_DOCKER_PRE_RUN_COMMANDS' in env:
        for pre_run_cmd in env['CM_DOCKER_PRE_RUN_COMMANDS']:
            f.write('RUN ' + pre_run_cmd + EOL)

    run_cmd_extra = " " + \
        env.get('CM_DOCKER_RUN_CMD_EXTRA', '').replace(":", "=")
    gh_token = get_value(env, config, "GH_TOKEN", "CM_GH_TOKEN")
    if gh_token:
        run_cmd_extra = " --env.CM_GH_TOKEN=$CM_GH_TOKEN"

    f.write(EOL + '# Run commands' + EOL)
    for comment in env.get('CM_DOCKER_RUN_COMMENTS', []):
        f.write(comment + EOL)

    skip_extra = False
    if 'CM_DOCKER_RUN_CMD' not in env:
        env['CM_DOCKER_RUN_CMD'] = ""
        if 'CM_DOCKER_RUN_SCRIPT_TAGS' not in env:
            env['CM_DOCKER_RUN_CMD'] += "cm version"
            skip_extra = True
        else:
            if str(env.get('CM_DOCKER_NOT_PULL_UPDATE', 'False')
                   ).lower() not in ["yes", "1", "true"]:
                env['CM_DOCKER_RUN_CMD'] += "cm pull repo && "
            env['CM_DOCKER_RUN_CMD'] += "cm run script --tags=" + \
                env['CM_DOCKER_RUN_SCRIPT_TAGS'] + ' --quiet'
    else:
        if str(env.get('CM_DOCKER_NOT_PULL_UPDATE', 'False')
               ).lower() not in ["yes", "1", "true"]:
            env['CM_DOCKER_RUN_CMD'] = "cm pull repo && " + \
                env['CM_DOCKER_RUN_CMD']

    print(env['CM_DOCKER_RUN_CMD'])
    fake_run = env.get("CM_DOCKER_FAKE_RUN_OPTION",
                       " --fake_run") + dockerfile_env_input_string
    fake_run = fake_run + \
        " --fake_deps" if env.get('CM_DOCKER_FAKE_DEPS') else fake_run

    x = 'RUN ' + env['CM_DOCKER_RUN_CMD']

    if not skip_extra:
        x += fake_run
        if '--quiet' not in x:
            x += ' --quiet'
        if run_cmd_extra != '':
            x += ' ' + run_cmd_extra

    if env.get('CM_DOCKER_RUN_SCRIPT_TAGS', '') != '' and str(env.get(
            'CM_DOCKER_ADD_DEPENDENT_SCRIPTS_RUN_COMMANDS', '')).lower() in ["yes", "1", "true"]:
        cm_input = {'action': 'run',
                    'automation': 'script',
                    'tags': f"""{env['CM_DOCKER_RUN_SCRIPT_TAGS']}""",
                    'print_deps': True,
                    'quiet': True,
                    'silent': True,
                    'fake_run': True,
                    'fake_deps': True
                    }
        r = self_module.cmind.access(cm_input)
        if r['return'] > 0:
            return r
        print_deps = r['new_state']['print_deps']
        fake_run_str = " --fake_run" if env.get('CM_DOCKER_FAKE_DEPS') else ""
        cmds = ["RUN " + dep for dep in print_deps]
        for cmd in cmds:
            f.write(cmd + fake_run_str + EOL)

    f.write(x + EOL)

    # fake_run to install the dependent scripts and caching them
    if not "run" in env['CM_DOCKER_RUN_CMD'] and str(
            env.get('CM_REAL_RUN', False)).lower() in ["false", "0", "no"]:
        fake_run = dockerfile_env_input_string

        x = 'RUN ' + env['CM_DOCKER_RUN_CMD'] + fake_run + run_cmd_extra
        if '--quiet' not in x:
            x += ' --quiet '
        x += EOL

        f.write(x)

    if 'CM_DOCKER_POST_RUN_COMMANDS' in env:
        for post_run_cmd in env['CM_DOCKER_POST_RUN_COMMANDS']:
            f.write('RUN ' + post_run_cmd + EOL)

    post_file = env.get('DOCKER_IMAGE_POST_FILE', '')
    if post_file != '':
        r = utils.load_txt(post_file)
        if r['return'] > 0:
            return r

        s = r['string']
        f.write(s + EOL)

    print(f"""Dockerfile written at {dockerfile_with_path}""")

    f.close()

    # f = open(env['CM_DOCKERFILE_WITH_PATH'], "r")
    # print(f.read())

    return {'return': 0}


def get_value(env, config, key, env_key=None):
    if not env_key:
        env_key = key

    if env.get(env_key, None) is not None:
        return env[env_key]

    docker_os = env['CM_DOCKER_OS']
    docker_os_version = env['CM_DOCKER_OS_VERSION']

    version_meta = config['distros'][docker_os]['versions'].get(
        docker_os_version, '')
    if key in version_meta:
        return version_meta[key]

    distro_meta = config['distros'][docker_os]
    if key in distro_meta:
        return distro_meta[key]

    if key in config:
        return config[key]

    return None



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

# Developer: Grigori Fursin

from cmind import utils
import os
import shutil


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    if 'CM_ML_MODEL_FILE_WITH_PATH' not in env:
        return {
            'return': 1, 'error': 'Please select a variation specifying the model to run'}

    run_opts = env.get('CM_RUN_OPTS', '')

    if env.get('CM_MLPERF_BACKEND', '') != '':
        run_opts += " -b " + env['CM_MLPERF_BACKEND']

    if env.get('CM_MLPERF_RUNNER', '') != '':
        run_opts += " -r " + env['CM_MLPERF_RUNNER']

    if env.get('CM_MLPERF_CONCURRENCY', '') != '':
        run_opts += " --concurrency " + env['CM_MLPERF_CONCURRENCY']

    if env.get('CM_MLPERF_EXECUTION_PROVIDER', '') != '':
        run_opts += " --ep " + env['CM_MLPERF_EXECUTION_PROVIDER']

    if env.get('CM_MLPERF_INTRAOP', '') != '':
        run_opts += " --intraop " + env['CM_MLPERF_INTRAOP']

    if env.get('CM_MLPERF_INTEROP', '') != '':
        run_opts += " --interop " + env['CM_MLPERF_INTEROP']

    if env.get('CM_MLPERF_EXECMODE', '') != '':
        run_opts += " --execmode " + env['CM_MLPERF_EXECUTION_MODE']

    if env.get('CM_MLPERF_LOADGEN_SAMPLES', '') != '':
        run_opts += " --samples " + env['CM_MLPERF_LOADGEN_SAMPLES']

    if env.get('CM_MLPERF_LOADGEN_EXPECTED_QPS', '') != '':
        run_opts += " --loadgen_expected_qps " + \
            env['CM_MLPERF_LOADGEN_EXPECTED_QPS']

    if env.get('CM_MLPERF_LOADGEN_DURATION_SEC', '') != '':
        run_opts += " --loadgen_duration_sec " + \
            env['CM_MLPERF_LOADGEN_DURATION_SEC']

    if env.get('CM_MLPERF_OUTPUT_DIR', '') != '':
        run_opts += " --output " + env['CM_MLPERF_OUTPUT_DIR']

    if env.get('CM_ML_MODEL_CODE_WITH_PATH', '') != '':
        run_opts += " --model_code " + env['CM_ML_MODEL_CODE_WITH_PATH']

    if env.get('CM_ML_MODEL_CFG_WITH_PATH', '') != '':
        run_opts += " --model_cfg " + env['CM_ML_MODEL_CFG_WITH_PATH']
    else:
        # Check cfg from command line
        cfg = env.get('CM_ML_MODEL_CFG', {})
        if len(cfg) > 0:
            del (env['CM_ML_MODEL_CFG'])

            import json
            import tempfile
            tfile = tempfile.NamedTemporaryFile(mode="w+", suffix='.json')

            fd, tfile = tempfile.mkstemp(suffix='.json', prefix='cm-cfg-')
            os.close(fd)

            with open(tfile, 'w') as fd:
                json.dump(cfg, fd)

            env['CM_APP_LOADGEN_GENERIC_PYTHON_TMP_CFG_FILE'] = tfile

            run_opts += " --model_cfg " + tfile

    if env.get('CM_ML_MODEL_SAMPLE_WITH_PATH', '') != '':
        run_opts += " --model_sample_pickle " + \
            env['CM_ML_MODEL_SAMPLE_WITH_PATH']

    # Add path to file model weights at the end of command line

    run_opts += ' ' + env['CM_ML_MODEL_FILE_WITH_PATH']

    env['CM_RUN_OPTS'] = run_opts

    print('')
    print('Assembled flags: {}'.format(run_opts))
    print('')

    return {'return': 0}


def postprocess(i):

    env = i['env']

    tfile = env.get('CM_APP_LOADGEN_GENERIC_PYTHON_TMP_CFG_FILE', '')

    if tfile != '' and os.path.isfile(tfile):
        os.remove(tfile)

    return {'return': 0}



from cmind import utils
import os
import json
import shutil
import subprocess


def preprocess(i):

    os_info = i['os_info']
    env = i['env']
    state = i['state']
    script_path = i['run_script_input']['path']

    if env.get('CM_MLPERF_SKIP_RUN', '') == "yes":
        return {'return': 0}

    if env.get('CM_RUN_DOCKER_CONTAINER', '') == "yes":
        return {'return': 0}

    if env.get('CM_MLPERF_POWER', '') == "yes":
        power = "yes"
    else:
        power = "no"

    rerun = True if env.get("CM_RERUN", "") != '' else False

    if 'CM_MLPERF_LOADGEN_SCENARIO' not in env:
        env['CM_MLPERF_LOADGEN_SCENARIO'] = "Offline"

    if 'CM_MLPERF_LOADGEN_MODE' not in env:
        env['CM_MLPERF_LOADGEN_MODE'] = "accuracy"

    if 'CM_MODEL' not in env:
        return {
            'return': 1, 'error': "Please select a variation specifying the model to run"}

    # if env['CM_MODEL'] == "resnet50":
    #    cmd = "cp " + os.path.join(env['CM_DATASET_AUX_PATH'], "val.txt") + " " + os.path.join(env['CM_DATASET_PATH'],
    #    "val_map.txt")
    #    ret = os.system(cmd)

    env['CM_MLPERF_LOADGEN_EXTRA_OPTIONS'] = " " + \
        env.get('CM_MLPERF_LOADGEN_EXTRA_OPTIONS', '') + " "

    if 'CM_MLPERF_LOADGEN_QPS' not in env:
        env['CM_MLPERF_LOADGEN_QPS_OPT'] = ""
    else:
        env['CM_MLPERF_LOADGEN_QPS_OPT'] = " --qps " + \
            env['CM_MLPERF_LOADGEN_QPS']

    env['CM_MLPERF_LOADGEN_EXTRA_OPTIONS'] += env['CM_MLPERF_LOADGEN_QPS_OPT']

    if 'CM_NUM_THREADS' not in env:
        if 'CM_MINIMIZE_THREADS' in env:
            env['CM_NUM_THREADS'] = str(int(env['CM_HOST_CPU_TOTAL_CORES']) //
                                        (int(env.get('CM_HOST_CPU_SOCKETS', '1')) * int(env.get('CM_HOST_CPU_TOTAL_CORES', '1'))))
        else:
            env['CM_NUM_THREADS'] = env.get('CM_HOST_CPU_TOTAL_CORES', '1')

    if env.get('CM_MLPERF_LOADGEN_MAX_BATCHSIZE', '') != '' and not env.get(
            'CM_MLPERF_MODEL_SKIP_BATCHING', False):
        env['CM_MLPERF_LOADGEN_EXTRA_OPTIONS'] += " --max-batchsize " + \
            str(env['CM_MLPERF_LOADGEN_MAX_BATCHSIZE'])

    if env.get('CM_MLPERF_LOADGEN_BATCH_SIZE', '') != '':
        env['CM_MLPERF_LOADGEN_EXTRA_OPTIONS'] += " --batch-size " + \
            str(env['CM_MLPERF_LOADGEN_BATCH_SIZE'])

    if env.get('CM_MLPERF_LOADGEN_QUERY_COUNT', '') != '' and not env.get(
            'CM_TMP_IGNORE_MLPERF_QUERY_COUNT', False) and env.get('CM_MLPERF_RUN_STYLE', '') != "valid":
        env['CM_MLPERF_LOADGEN_EXTRA_OPTIONS'] += " --count " + \
            env['CM_MLPERF_LOADGEN_QUERY_COUNT']

    print("Using MLCommons Inference source from '" +
          env['CM_MLPERF_INFERENCE_SOURCE'] + "'")

    if 'CM_MLPERF_CONF' not in env:
        env['CM_MLPERF_CONF'] = os.path.join(
            env['CM_MLPERF_INFERENCE_SOURCE'], "mlperf.conf")

    x = "" if os_info['platform'] == 'windows' else "'"
    if "llama2-70b" in env['CM_MODEL']:
        env['CM_MLPERF_LOADGEN_EXTRA_OPTIONS'] += " --mlperf-conf " + \
            x + env['CM_MLPERF_CONF'] + x
    else:
        env['CM_MLPERF_LOADGEN_EXTRA_OPTIONS'] += " --mlperf_conf " + \
            x + env['CM_MLPERF_CONF'] + x

    env['MODEL_DIR'] = env.get('CM_ML_MODEL_PATH')
    if not env['MODEL_DIR']:
        env['MODEL_DIR'] = os.path.dirname(
            env.get(
                'CM_MLPERF_CUSTOM_MODEL_PATH',
                env.get('CM_ML_MODEL_FILE_WITH_PATH')))

    RUN_CMD = ""

    scenario = env['CM_MLPERF_LOADGEN_SCENARIO']
    scenario_extra_options = ''

    NUM_THREADS = env['CM_NUM_THREADS']
    if int(NUM_THREADS) > 2 and env['CM_MLPERF_DEVICE'] == "gpu":
        NUM_THREADS = "2"  # Don't use more than 2 threads when run on GPU

    if env['CM_MODEL'] in ['resnet50', 'retinanet', 'stable-diffusion-xl']:
        scenario_extra_options += " --threads " + NUM_THREADS

    ml_model_name = env['CM_MODEL']
    if 'CM_MLPERF_USER_CONF' in env:
        user_conf_path = env['CM_MLPERF_USER_CONF']
        x = "" if os_info['platform'] == 'windows' else "'"
        scenario_extra_options += " --user_conf " + x + user_conf_path + x

    mode = env['CM_MLPERF_LOADGEN_MODE']
    mode_extra_options = ""

    # Grigori blocked for ABTF to preprocess data set on the fly for now
    # we can later move it to a separate script to preprocess data set

#    if 'CM_DATASET_PREPROCESSED_PATH' in env and env['CM_MODEL'] in  [ 'resnet50', 'retinanet' ]:
#        #dataset_options = " --use_preprocessed_dataset --preprocessed_dir "+env['CM_DATASET_PREPROCESSED_PATH']
#        if env.get('CM_MLPERF_LAST_RELEASE') not in [ "v2.0", "v2.1" ]:
#            dataset_options = " --use_preprocessed_dataset --cache_dir "+env['CM_DATASET_PREPROCESSED_PATH']
#        else:
#            dataset_options = ""
#        if env['CM_MODEL'] == "retinanet":
#            dataset_options += " --dataset-list "+ env['CM_DATASET_ANNOTATIONS_FILE_PATH']
#        elif env['CM_MODEL'] == "resnet50":
#            dataset_options += " --dataset-list "+ os.path.join(env['CM_DATASET_AUX_PATH'], "val.txt")
#        env['DATA_DIR'] = env.get('CM_DATASET_PREPROCESSED_PATH')
#    else:
#        if 'CM_DATASET_PREPROCESSED_PATH' in env:
#            env['DATA_DIR'] = env.get('CM_DATASET_PREPROCESSED_PATH')
#        else:
#            env['DATA_DIR'] = env.get('CM_DATASET_PATH')
#        dataset_options = ''

    # Grigori added for ABTF
#    dataset_path = env.get('CM_DATASET_PATH')
#    env['DATA_DIR'] = dataset_path

#    dataset_options = " --dataset-list " + env['CM_DATASET_ANNOTATIONS_FILE_PATH']
#    dataset_options += " --cache_dir " + os.path.join(script_path, 'preprocessed-dataset')

    dataset_options = ''

    if env.get('CM_MLPERF_EXTRA_DATASET_ARGS', '') != '':
        dataset_options += " " + env['CM_MLPERF_EXTRA_DATASET_ARGS']

    if mode == "accuracy":
        mode_extra_options += " --accuracy"
        env['CM_OUTPUT_PREDICTIONS_PATH'] = os.path.join(
            env['CM_DATASET_MLCOMMONS_COGNATA_PATH'],
            env['CM_DATASET_MLCOMMONS_COGNATA_SERIAL_NUMBERS'],
            'Cognata_Camera_01_8M_png',
            'output')

    elif mode == "performance":
        pass

    elif mode == "compliance":

        audit_full_path = env['CM_MLPERF_INFERENCE_AUDIT_PATH']
        mode_extra_options = " --audit '" + audit_full_path + "'"

    if env.get('CM_MLPERF_OUTPUT_DIR', '') == '':
        env['CM_MLPERF_OUTPUT_DIR'] = os.getcwd()

    mlperf_implementation = env.get('CM_MLPERF_IMPLEMENTATION', 'reference')

    # Generate CMD

    # Grigori updated for ABTF demo
#    cmd, run_dir = get_run_cmd(os_info, env, scenario_extra_options, mode_extra_options, dataset_options, mlperf_implementation)
    cmd, run_dir = get_run_cmd_reference(
        os_info, env, scenario_extra_options, mode_extra_options, dataset_options, script_path)

    if env.get('CM_NETWORK_LOADGEN', '') == "lon":

        run_cmd = i['state']['mlperf_inference_run_cmd']
        env['CM_SSH_RUN_COMMANDS'] = []
        env['CM_SSH_RUN_COMMANDS'].append(
            run_cmd.replace(
                "--network=lon",
                "--network=sut") + " &")

    env['CM_MLPERF_RUN_CMD'] = cmd
    env['CM_RUN_DIR'] = run_dir
    env['CM_RUN_CMD'] = cmd
    env['CK_PROGRAM_TMP_DIR'] = env.get('CM_ML_MODEL_PATH')  # for tvm

    if env.get('CM_HOST_PLATFORM_FLAVOR', '') == "arm64":
        env['CM_HOST_PLATFORM_FLAVOR'] = "aarch64"

    if not env.get('CM_COGNATA_ACCURACY_DUMP_FILE'):
        env['CM_COGNATA_ACCURACY_DUMP_FILE'] = os.path.join(
            env['OUTPUT_DIR'], "accuracy.txt")

    return {'return': 0}


def get_run_cmd_reference(os_info, env, scenario_extra_options,
                          mode_extra_options, dataset_options, script_path=None):

    q = '"' if os_info['platform'] == 'windows' else "'"

    ##########################################################################
    # Grigori added for ABTF demo

    if env['CM_MODEL'] in ['retinanet']:

        run_dir = os.path.join(script_path, 'ref')

        env['RUN_DIR'] = run_dir

        env['OUTPUT_DIR'] = env['CM_MLPERF_OUTPUT_DIR']

        cognata_dataset_path = env['CM_DATASET_MLCOMMONS_COGNATA_PATH']
# cognata_dataset_path = env['CM_DATASET_PATH'] # Using open images
# dataset for some tests

        path_to_model = env.get(
            'CM_MLPERF_CUSTOM_MODEL_PATH',
            env.get(
                'CM_ML_MODEL_FILE_WITH_PATH',
                env.get('CM_ML_MODEL_CODE_WITH_PATH')))
        env['MODEL_FILE'] = path_to_model

        cmd = env['CM_PYTHON_BIN_WITH_PATH'] + " " + os.path.join(run_dir, "python", "main.py") + " --profile " + env['CM_MODEL'] + "-" + env['CM_MLPERF_BACKEND'] + \
            " --model=" + q + path_to_model + q + \
            " --dataset=" + env["CM_MLPERF_VISION_DATASET_OPTION"] + \
            " --dataset-path=" + q + cognata_dataset_path + q + \
            " --cache_dir=" + q + os.path.join(script_path, 'tmp-preprocessed-dataset') + q + \
            " --scenario " + env['CM_MLPERF_LOADGEN_SCENARIO'] + " " + \
            " --output " + q + env['OUTPUT_DIR'] + q + " " + \
            env['CM_MLPERF_LOADGEN_EXTRA_OPTIONS'] + \
            scenario_extra_options + mode_extra_options + dataset_options

    ##########################################################################

    return cmd, run_dir


def postprocess(i):

    env = i['env']

    state = i['state']

    inp = i['input']

    return {'return': 0}



from __future__ import print_function

import os.path
import os
import csv
from google.auth.transport.requests import Request
from google.oauth2.credentials import Credentials
from google_auth_oauthlib.flow import InstalledAppFlow
from googleapiclient.discovery import build
from googleapiclient.errors import HttpError

# If modifying these scopes, delete the file token.json.
SCOPES = ['https://www.googleapis.com/auth/spreadsheets']

# The ID of a sample document.
DOCUMENT_ID = os.environ['CM_GOOGLE_SPREADSHEET_ID']


def main():
    """Shows basic usage of the Docs API.
    Prints the title of a sample document.
    """
    creds = None
    # The file token.json stores the user's access and refresh tokens, and is
    # created automatically when the authorization flow completes for the first
    # time.
    if os.path.exists('token.json'):
        creds = Credentials.from_authorized_user_file('token.json', SCOPES)
    # If there are no (valid) credentials available, let the user log in.
    if not creds or not creds.valid:
        if creds and creds.expired and creds.refresh_token:
            creds.refresh(Request())
        else:
            flow = InstalledAppFlow.from_client_secrets_file(
                'credentials.json', SCOPES)
            creds = flow.run_local_server(port=0)
        # Save the credentials for the next run
        with open('token.json', 'w') as token:
            token.write(creds.to_json())

    try:
        service = build("sheets", "v4", credentials=creds)
        sheet_name = os.environ.get('CM_GOOGLE_SHEET_NAME', 'Sheet1')
        csv_file = os.environ['CM_CSV_FILE_PATH']

        f = open(csv_file, "r")
        values = [r for r in csv.reader(f)]
        request = service.spreadsheets().values().update(
            spreadsheetId=DOCUMENT_ID,
            range=sheet_name,
            valueInputOption="USER_ENTERED",
            body={
                "values": values}).execute()

    except HttpError as err:
        print(err)


if __name__ == '__main__':
    main()



# Developer(s): Grigori Fursin

import cmind
import os
import datetime
import misc


def page(st, params):

    current_script_path = os.environ.get('CM_TMP_CURRENT_SCRIPT_PATH', '')

    url_prefix = st.config.get_option('server.baseUrlPath') + '/'

    name = params.get('name', [''])[0].strip()
    tags = params.get('tags', [''])[0].lower()

    readme = os.path.join(current_script_path, 'playground_beta_README.md')

    md = ''

    if os.path.isfile(readme):

        r = cmind.utils.load_txt(readme)
        if r['return'] > 0:
            return r

        md += r['string']

    md = md.replace('{{URL_PREFIX}}', url_prefix)

#    st.markdown(md)
    st.write(md, unsafe_allow_html=True)

    end_html = ''

    return {'return': 0, 'end_html': end_html}



from cmind import utils
import os


def preprocess(i):
    env = i['env']
    if env.get('CM_HF_TOKEN', '') != '':
        env['CM_HF_LOGIN_CMD'] = f"""git config --global credential.helper store && huggingface-cli login --token {env['CM_HF_TOKEN']} --add-to-git-credential
"""
    elif str(env.get('CM_HF_DO_LOGIN')).lower() in ["yes", "1", "true"]:
        env['CM_HF_LOGIN_CMD'] = f"""git config --global credential.helper store && huggingface-cli login
"""
    return {'return': 0}


def postprocess(i):
    env = i['env']

    r = i['automation'].parse_version({'match_text': r'huggingface_hub\s*version:\s*([\d.]+)',
                                       'group_number': 1,
                                       'env_key': 'CM_GITHUBCLI_VERSION',
                                       'which_env': i['env']})
    if r['return'] > 0:
        return r

    version = r['version']

    print(i['recursion_spaces'] + '    Detected version: {}'.format(version))

    return {'return': 0, 'version': version}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import shutil


def preprocess(i):

    env = i['env']

    if env.get('CM_CNNDM_INTEL_VARIATION', '') == 'yes':
        i['run_script_input']['script_name'] = "run-intel"
    else:
        print("Using MLCommons Inference source from '" +
              env['CM_MLPERF_INFERENCE_SOURCE'] + "'")

    return {'return': 0}


def postprocess(i):
    env = i['env']

    if env.get('CM_DATASET_CALIBRATION', '') == "no":
        env['CM_DATASET_PATH'] = os.path.join(os.getcwd(), 'install')
        env['CM_DATASET_EVAL_PATH'] = os.path.join(
            os.getcwd(), 'install', 'cnn_eval.json')
        env['CM_DATASET_CNNDM_EVAL_PATH'] = os.path.join(
            os.getcwd(), 'install', 'cnn_eval.json')
        env['CM_GET_DEPENDENT_CACHED_PATH'] = env['CM_DATASET_PATH']
    else:
        env['CM_CALIBRATION_DATASET_PATH'] = os.path.join(
            os.getcwd(), 'install', 'cnn_dailymail_calibration.json')
        env['CM_CALIBRATION_DATASET_CNNDM_PATH'] = os.path.join(
            os.getcwd(), 'install', 'cnn_dailymail_calibration.json')
        env['CM_GET_DEPENDENT_CACHED_PATH'] = env['CM_CALIBRATION_DATASET_PATH']

    return {'return': 0}



# Developer(s): Grigori Fursin

import cmind
import os
import datetime
import misc


def page(st, params):

    url_prefix = st.config.get_option('server.baseUrlPath') + '/'

    url_scripts = url_prefix + '?action=scripts'
    url_contributors = url_prefix + '?action=contributors'

    name = params.get('name', [''])[0].strip()
    tags = params.get('tags', [''])[0].lower()

    ii = {'action': 'find',
          'automation': 'challenge,3d84abd768f34e08'}

    if name != '':
        ii['artifact'] = name
    if tags != '':
        ii['tags'] = tags

    r = cmind.access(ii)
    if r['return'] > 0:
        return r

    lst = r['list']

    end_html = ''

    if len(lst) == 0:
        st.markdown('Challenges were not found!')
    else:
        artifact = None

        if len(lst) == 1:
            artifact = lst[0]
        else:
            challenges = []

            date_now = datetime.datetime.now().isoformat()
            date_now2 = int(date_now[0:4] + date_now[5:7] + date_now[8:10])

            ongoing = []

            for l in sorted(lst, key=lambda x: (
                -int(x.meta.get('date_open', '0')),
                -int(x.meta.get('date_close', '0')),
                x.meta.get('title', '')
            )):

                row = {}

                meta = l.meta
                row['uid'] = meta['uid']

                name = meta.get('title', meta['alias'])

                row['name'] = name

                if meta.get('hot', False):
                    row['hot'] = True

                for k in ['date_close_extension', 'points',
                          'trophies', 'prize', 'prize_short', 'skip', 'sort']:
                    if k in meta:
                        row[k] = meta[k]

                under_preparation = meta.get('under_preparation', False)
                row['under_preparation'] = under_preparation

                date_open = meta.get('date_open', '')
                date_close = meta.get('date_close', '')

                s_date_open = ''
                if date_open != '':
                    r = misc.convert_date(date_open)
                    s_date_open = r['string'] if r['return'] == 0 else ''

                    row['orig_date_open'] = date_open
                    row['date_open'] = s_date_open

                s_date_close = ''
                if date_close != '':
                    r = misc.convert_date(date_close)
                    s_date_close = r['string'] if r['return'] == 0 else ''

                    row['orig_date_close'] = date_close
                    row['date_close'] = s_date_close

                diff1 = 0
                diff2 = 0

                if date_open != '':
                    diff1 = int(date_open) - int(date_now2)

                if date_close != '':
                    diff2 = int(date_close) - int(date_now2)

                prefix = ''
                if under_preparation:
                    prefix = 'Under preparation: '
                else:
                    if date_open != '' and diff1 > 0:
                        prefix = 'Opens on {}: '.format(s_date_open)
                    else:
                        if date_close != '':
                            if diff2 < 0:
                                prefix = 'Finished on {}: '.format(
                                    s_date_close)
                            else:
                                prefix = 'Open and finishes on {}: '.format(
                                    s_date_close)
                        else:
                            prefix = 'Open: '.format(s_date_close)

                # Check if open challenge even if under preparation
                if date_open and (date_close == '' or (
                        diff1 <= 0 and diff2 > 0)):
                    ongoing.append(row)
                else:
                    challenges.append(
                        {'prefix': prefix, 'name': name, 'uid': l.meta['uid']})

            # Show ongoing if open
            if len(ongoing) > 0:

                # Check hot
                hot = []
                ongoing_without_hot = []

                for row in ongoing:
                    if row.get('hot', False):
                        hot.append(row)
                    else:
                        ongoing_without_hot.append(row)

                # Some info
                x = '''
                     <i>
                     <small>
                     Collaborative benchmarking and optimization of AI applications and systems
                     (latency, throughput, power consumption, accuracy, costs ...)
                     is organized by <a href="https://github.com/mlcommons/ck/blob/master/docs/taskforce.md">MLCommons</a>,
                     <a href="https://cKnowledge.org">cKnowledge</a>
                     and the <a href="https://cTuning.org">cTuning foundation</a>
                     and powered by <a href="{}">Collective Mind automation recipes</a>.
                     We deeply thank all our <a href="{}">participants and contributors</a>!
                     </small>
                     </i>
                      <br>
                      <br>
                    '''.format(url_scripts, url_contributors)
                st.write(x, unsafe_allow_html=True)

                # Check if hot
                if len(hot) > 0:
                    st.markdown('#### Hot challenges')

                    md_tmp = ''

                    for row in sorted(hot, key=lambda row: (int(row.get('orig_date_close', 9999999999)),
                                                            row.get('sort', 0),
                                                            row.get(
                                                                'name', ''),
                                                            row.get(
                                                                'under_preparation', False)
                                                            )):
                        x = row['name']
                        x = x[0].upper() + x[1:]

                        url = url_prefix + \
                            '?action=challenges&name={}'.format(row['uid'])

                        date_close = row.get('date_close', '').strip()
                        y = ' (Closing date: **{}**)'.format(
                            date_close) if date_close != '' else ''

                        md_tmp += '* [{}]({}){}\n'.format(x, url, y)

                    st.markdown(md_tmp)

                    st.markdown('#### On-going challenges')

                # Continue all
                ind = 1

                data = []

                for row in sorted(ongoing_without_hot, key=lambda row: (int(row.get('orig_date_close', 9999999999)),
                                                                        row.get(
                                                                            'sort', 0),
                                                                        row.get(
                                                                            'name', ''),
                                                                        row.get(
                                                                            'under_preparation', False)
                                                                        )):
                    if row.get('skip', False):
                        continue

                    xrow = []

                    md = ''
                    up = row.get('under_preparation', False)

                    x = row['name']
                    y = ''
                    if up:
                        x = x[0].lower() + x[1:]
                        y = '<i>Under preparation:</i> '

                    url = url_prefix + \
                        '?action=challenges&name={}'.format(row['uid'])
# md += '###### {}) {}[{}]({})\n'.format(str(ind), y, x, url)

                    x = '''
                         <div style="">
                          <b>
                          {}<a href="{}">{}</a>
                          </b>
                        </div>
                        '''.format(y, url, x).replace('\n', '')
#                    st.write(x, unsafe_allow_html = True)

                    xrow.append(x)

                    # Assemble info
                    x = ''

                    date_close = row.get('date_close', '')
                    y = ''
                    if date_close != '' and date_close is not None:
                        x += '&nbsp;&nbsp;&nbsp;Closing date: **{}**\n'.format(
                            date_close)
                        y = date_close.replace(' ', '&nbsp;')

                    xrow.append(y)

                    y = ''
                    if row.get('date_close_extension', False):
                        y = 'until done'

                    xrow.append(y)

#                    points = row.get('points',0)
#                    y = ''
#                    if points>0:
#                        x += ' &nbsp;&nbsp;Points: **{}**\n'.format(str(points))
#                        y = str(points)
#
#                    xrow.append(y)

                    awards = ''

                    trophies = row.get('trophies', False)
                    if trophies:
                        x += ' &nbsp;&nbsp;Trophy: **Yes**\n'
                        awards += '&#127942'


#                    prize = row.get('prize_short','')
#                    if prize!='':
#                        x += ' &nbsp;&nbsp;Prizes from [MLCommons organizations](https://mlcommons.org), [cTuning foundation](https://cTuning.org) and [cKnowledge.org](https:/cKnowledge.org): **{}**\n'.format(prize)
#                        if awards!='': awards+=' , '
#                        awards += prize
#
#                    xrow.append(awards)

                    if x != '':
                        md += '&nbsp;&nbsp;&nbsp;&nbsp; ' + x

#                    st.markdown(md)

                    data.append(xrow)
                    ind += 1

                import pandas as pd
                import numpy as np

                df = pd.DataFrame(data,
                                  columns=['Challenge', 'Closing&nbsp;date', 'Extension'])

                df.index += 1

#                st.table(df)
                st.write(
                    df.to_html(
                        escape=False,
                        justify='left'),
                    unsafe_allow_html=True)

        # Show selector for all
#        challenge = st.selectbox('View past benchmarking, optimization, reproducibility and replicability challenges:',
#                                 range(len(challenges)),
#                                 format_func=lambda x: challenges[x],
#                                 index=0, key='challenge')
#
#        if challenge>0:
#            artifact = artifacts[challenge]

        # Process 1 challenge
        if artifact is None:
            #            st.markdown('#### Past or future challenges:')

            st.markdown('#### Future or past challenges')

            for c in challenges:

                prefix = c['prefix']
                name = c['name']
                uid = c['uid']

                url = url_prefix + '?action=challenges&name={}'.format(uid)

                x = '''
                     <div style="">
                      {}) {}<a href="{}">{}</a>
                    </div>
                    '''.format(str(ind), prefix, url, name)

                st.write(x, unsafe_allow_html=True)

                ind += 1

        else:
            meta = artifact.meta

            name = meta.get('title', meta['alias'])
            uid = meta['uid']

            st.write('''
             <center>
              <h4>Challenge: {}</h4>
             </center>
             '''.format(name),
                     unsafe_allow_html=True
                     )

            end_html = '<center><small><i><a href="{}">Self link</a></i></small></center>'.format(
                misc.make_url(meta['uid'], action='challenges', md=False))

            # Check basic password
            password_hash = meta.get('password_hash', '')
            view = True
            if password_hash != '':
                view = False

                password = st.text_input(
                    "Enter password", type="password", key="password")

                if password != '':
                    import bcrypt
                    # TBD: temporal hack to demo password protection for
                    # experiments
                    password_salt = b'$2b$12$ionIRWe5Ft7jkn4y/7C6/e'
                    password_hash2 = bcrypt.hashpw(
                        password.encode('utf-8'), password_salt)

                    if password_hash.encode('utf-8') == password_hash2:
                        view = True
                    else:
                        st.markdown('**Warning:** wrong password')

            if not view:
                return {'return': 0, 'end_html': end_html}

            z = ''
            date_open = meta.get('date_open', '')
            if date_open != '':
                # Format YYYYMMDD
                r = misc.convert_date(date_open)
                if r['return'] > 0:
                    return r
                z += '* **Open date:** {}\n'.format(r['string'])

            date_close = meta.get('date_close', '')
            if date_close != '':
                # Format YYYYMMDD
                r = misc.convert_date(date_close)
                if r['return'] > 0:
                    return r
                z += '* **Closing date:** {}\n'.format(r['string'])

            if meta.get('trophies', False):
                z += '* **MLCommons Collective Knowledge Contributor award:** Yes\n'

            prize_short = meta.get('prize_short', '')
            if prize_short != '':
                z += '* **Prizes:** {}\n'.format(prize_short)

#            prize = meta.get('prize','')
#            if prize!='':
#                z+='* **Student prizes:** {}\n'.format(prize)

            urls = meta.get('urls', [])
            url = meta.get('url', '')

            if url != '':
                urls.append(url)

            if len(urls) > 0:
                x = '* **External link:** '
                md = ''
                if len(urls) > 1:
                    md = '* **External links:**\n'
                    x = '   * '

                for u in urls:
                    md += x + '[{}]({})\n'.format(u, u)
                z += md + '\n'

            # Check if has linked experiments
            experiments = meta.get('experiments', [])

            if len(experiments) > 0:
                md = '* **Shared experiments:**\n'

                for e in experiments:
                    tags = e.get('tags', '')
                    name = e.get('name', '')

                    if tags != '':
                        md += '  * ' + \
                            misc.make_url(
                                tags, action='experiments', key='tags') + '\n'
                    elif name != '':
                        md += '  * ' + \
                            misc.make_url(name, action='experiments') + '\n'

                z += md + '\n'

            st.markdown(z)

            # Check if has text
            path = artifact.path

            for f in ['README.md', 'info.html']:
                f1 = os.path.join(path, f)
                if os.path.isfile(f1):
                    r = cmind.utils.load_txt(f1)
                    if r['return'] > 0:
                        return r

                    s = r['string']

                    st.markdown('---')

                    if f.endswith('.html'):
                        y = s.split('\n')
                        ss = ''
                        for x in y:
                            ss += x.strip() + '\n'

                        st.write(ss, unsafe_allow_html=True)
                    else:
                        st.markdown(s)

                    break

            # Check associated reports
            r = cmind.access({'action': 'find',
                              'automation': 'report,6462ecdba2054467',
                             'tags': 'challenge-{}'.format(uid)})
            if r['return'] > 0:
                return r

            lst = r['list']

            for l in lst:
                report_path = l.path

                f1 = os.path.join(report_path, 'README.md')
                if os.path.isfile(f1):
                    report_meta = l.meta

                    report_alias = report_meta['alias']
                    report_title = report_meta.get('title', '')

                    report_name = report_title if report_title != '' else report_alias

                    r = cmind.utils.load_txt(f1)
                    if r['return'] > 0:
                        return r

                    s = r['string']

                    st.markdown('---')
                    st.markdown('### ' + report_name)

                    st.markdown(s, unsafe_allow_html=True)

    return {'return': 0, 'end_html': end_html}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils

import os
import json
import shutil
import subprocess
import copy
import cmind as cm
import platform
import sys
import mlperf_utils
import re
from datetime import datetime, timezone


def preprocess(i):

    env = i['env']
    state = i['state']

    if env.get('CM_MLPERF_IMPLEMENTATION', '') == 'nvidia':
        if env.get('CM_NVIDIA_GPU_NAME', '') in [
                "rtx_4090", "a100", "t4", "l4", "orin", "custom"]:
            env['CM_NVIDIA_HARNESS_GPU_VARIATION'] = "_" + \
                env['CM_NVIDIA_GPU_NAME']
            env['CM_NVIDIA_GPU_MEMORY'] = ''
        else:
            gpu_memory = i['state'].get(
                'cm_cuda_device_prop', '').get('Global memory')
            gpu_memory_size = str(
                int((float(gpu_memory) / (1024 * 1024 * 1024) + 7) / 8) * 8)
            env['CM_NVIDIA_GPU_MEMORY'] = gpu_memory_size
            env['CM_NVIDIA_HARNESS_GPU_VARIATION'] = ''

    if 'cmd' in i['input']:
        state['mlperf_inference_run_cmd'] = "cm run script " + \
            " ".join(i['input']['cmd'])

    state['mlperf-inference-implementation'] = {}

    run_state = i['run_script_input']['run_state']
    state['mlperf-inference-implementation']['script_id'] = run_state['script_id'] + \
        ":" + ",".join(run_state['script_variation_tags'])

    if env.get('CM_VLLM_SERVER_MODEL_NAME', '') != '' and env.get(
            'CM_ML_MODEL_FULL_NAME', '') == '':
        env['CM_ML_MODEL_FULL_NAME'] = env['CM_VLLM_SERVER_MODEL_NAME'].replace(
            "/", "_")

    return {'return': 0}


def postprocess(i):

    os_info = i['os_info']

    xsep = '^' if os_info['platform'] == 'windows' else '\\'

    env = i['env']
    inp = i['input']
    env['CMD'] = ''
    state = i['state']

    # if env.get('CM_MLPERF_USER_CONF', '') == '':
    #    return {'return': 0}

    output_dir = env['CM_MLPERF_OUTPUT_DIR']

    result_sut_folder_path = env['CM_MLPERF_INFERENCE_RESULTS_SUT_PATH']

    mode = env['CM_MLPERF_LOADGEN_MODE']

    if not os.path.exists(output_dir) or not os.path.exists(
            os.path.join(output_dir, "mlperf_log_summary.txt")):
        # No output, fake_run?
        return {'return': 0}

    # in power mode copy the log files from tmp_power directory
    if env.get('CM_MLPERF_POWER', '') == "yes" and mode == "performance":
        mlperf_power_logs_dir = os.path.join(
            env['CM_MLPERF_OUTPUT_DIR'], "..", "power")
        mlperf_ranging_logs_dir = os.path.join(
            env['CM_MLPERF_OUTPUT_DIR'], "..", "ranging")

        if os.path.exists(os.path.join(
                env['CM_MLPERF_POWER_LOG_DIR'], "power")):
            if os.path.exists(mlperf_power_logs_dir):
                shutil.rmtree(mlperf_power_logs_dir)
            shutil.copytree(
                os.path.join(
                    env['CM_MLPERF_POWER_LOG_DIR'],
                    "power"),
                mlperf_power_logs_dir)

        if os.path.exists(os.path.join(
                env['CM_MLPERF_POWER_LOG_DIR'], "ranging")):
            if os.path.exists(mlperf_ranging_logs_dir):
                shutil.rmtree(mlperf_ranging_logs_dir)
            shutil.copytree(
                os.path.join(
                    env['CM_MLPERF_POWER_LOG_DIR'],
                    "ranging"),
                mlperf_ranging_logs_dir)

        if os.path.exists(os.path.join(
                env['CM_MLPERF_POWER_LOG_DIR'], "run_1", "spl.txt")):
            shutil.copyfile(
                os.path.join(
                    env['CM_MLPERF_POWER_LOG_DIR'],
                    "run_1",
                    "spl.txt"),
                os.path.join(
                    env['CM_MLPERF_OUTPUT_DIR'],
                    "spl.txt"))

    model = env['CM_MODEL']
    model_full_name = env.get('CM_ML_MODEL_FULL_NAME', model)

    if mode == "accuracy" or mode == "compliance" and env[
            'CM_MLPERF_LOADGEN_COMPLIANCE_TEST'] == "TEST01":
        out_baseline_accuracy_string = f"""> {os.path.join(output_dir, "accuracy", "baseline_accuracy.txt")} """
        out_compliance_accuracy_string = f"""> {os.path.join(output_dir, "accuracy", "compliance_accuracy.txt")} """
        if model == "resnet50":
            accuracy_filename = "accuracy-imagenet.py"
            accuracy_filepath = os.path.join(env['CM_MLPERF_INFERENCE_CLASSIFICATION_AND_DETECTION_PATH'], "tools",
                                             accuracy_filename)
            dataset_args = " --imagenet-val-file " + \
                os.path.join(env['CM_DATASET_AUX_PATH'], "val.txt")
            accuracy_log_file_option_name = " --mlperf-accuracy-file "
            datatype_option = " --dtype " + env['CM_IMAGENET_ACCURACY_DTYPE']

        elif model == "retinanet":
            accuracy_filename = "accuracy-openimages.py"
            accuracy_filepath = os.path.join(env['CM_MLPERF_INFERENCE_CLASSIFICATION_AND_DETECTION_PATH'], "tools",
                                             accuracy_filename)
            dataset_args = " --openimages-dir " + \
                os.getcwd()  # just to make the script happy
            accuracy_log_file_option_name = " --mlperf-accuracy-file "
            datatype_option = ""

        elif 'bert' in model:
            accuracy_filename = "accuracy-squad.py"
            accuracy_filepath = os.path.join(
                env['CM_MLPERF_INFERENCE_BERT_PATH'], accuracy_filename)
            dataset_args = " --val_data '" + env['CM_DATASET_SQUAD_VAL_PATH'] + "' --vocab_file '" + \
                env['CM_DATASET_SQUAD_VOCAB_PATH'] + \
                "' --out_file predictions.json "
            accuracy_log_file_option_name = " --log_file "
            datatype_option = " --output_dtype " + \
                env['CM_SQUAD_ACCURACY_DTYPE']

        elif 'rgat' in model:
            accuracy_filename = "accuracy_igbh.py"
            accuracy_filepath = os.path.join(
                env['CM_MLPERF_INFERENCE_RGAT_PATH'], "tools", accuracy_filename)
            dataset_args = " --dataset-path '" + env['CM_DATASET_IGBH_PATH'] + "' --dataset-size '" + \
                env['CM_DATASET_IGBH_SIZE'] + "'"
            accuracy_log_file_option_name = " --mlperf-accuracy-file "
            datatype_option = ""
            out_baseline_accuracy_string = f""" --output-file {os.path.join(output_dir, "accuracy", "baseline_accuracy.txt")} """
            out_compliance_accuracy_string = f""" --output-file {os.path.join(output_dir, "accuracy", "compliance_accuracy.txt")} """

        elif 'stable-diffusion-xl' in model:
            pass  # No compliance check for now
        elif 'gpt' in model:
            pass  # No compliance check for now
        elif 'llama2-70b' in model:
            pass  # No compliance check for now
        elif 'mixtral-8x7b' in model:
            pass  # No compliance check for now
        else:
            pass  # Not giving an error now. But accuracy paths need to be done for other benchmarks which may need the non-determinism test
            # return {'return': 1, 'error': f'Accuracy paths not done for model
            # {model}'}
    scenario = env['CM_MLPERF_LOADGEN_SCENARIO']

    if not state.get('cm-mlperf-inference-results'):
        state['cm-mlperf-inference-results'] = {}
    if not state.get('cm-mlperf-inference-results-last'):
        state['cm-mlperf-inference-results-last'] = {}
    if not state['cm-mlperf-inference-results'].get(
            state['CM_SUT_CONFIG_NAME']):
        state['cm-mlperf-inference-results'][state['CM_SUT_CONFIG_NAME']] = {}
    if not state['cm-mlperf-inference-results'][state['CM_SUT_CONFIG_NAME']
                                                ].get(model):
        state['cm-mlperf-inference-results'][state['CM_SUT_CONFIG_NAME']][model] = {}
    if not state['cm-mlperf-inference-results'][state['CM_SUT_CONFIG_NAME']
                                                ][model].get(scenario):
        state['cm-mlperf-inference-results'][state['CM_SUT_CONFIG_NAME']
                                             ][model][scenario] = {}

    # if env.get("CM_MLPERF_FIND_PERFORMANCE_MODE", '') == "yes" and mode ==
    # "performance" and scenario != "Server":
    if mode == "performance" and scenario != "Server":
        os.chdir(output_dir)
        if not os.path.exists("mlperf_log_summary.txt"):
            return {'return': 0}

        if scenario in ["Offline", "Server"]:
            metric = "target_qps"
        elif scenario.endswith("Stream"):
            metric = "target_latency"
        else:
            return {'return': 1,
                    'error': 'Unsupported scenario: {}'.format(scenario)}

        import re
        import yaml
        pattern = {}
        pattern["Offline"] = "Samples per second: (.*)\n"
        pattern["SingleStream"] = "Mean latency \\(ns\\)\\s*:(.*)"
        pattern["MultiStream"] = "Mean latency \\(ns\\)\\s*:(.*)"
        print("\n")
        with open("mlperf_log_summary.txt", "r") as fp:
            summary = fp.read()

        result = re.findall(pattern[scenario], summary)

        if not result:
            return {
                'return': 1, 'error': f'No {metric} found in performance summary. Pattern checked "{pattern[metric]}"'}

        value = result[0].strip()
        if "\\(ns\\)" in pattern[scenario]:
            value = str(float(value) / 1000000)  # convert to milliseconds

        sut_name = state['CM_SUT_CONFIG_NAME']
        sut_config = state['CM_SUT_CONFIG'][sut_name]
        sut_config_path = state['CM_SUT_CONFIG_PATH'][sut_name]
        if scenario not in sut_config[model_full_name]:
            sut_config[model_full_name][scenario] = {}
        sut_config[model_full_name][scenario][metric] = value

        print(
            f"SUT: {sut_name}, model: {model_full_name}, scenario: {scenario}, {metric} updated as {value}")
        print(f"New config stored in {sut_config_path}")
        with open(sut_config_path, "w") as f:
            yaml.dump(sut_config, f)

    if mode in ["performance", "accuracy"]:
        # if measurements file exist read it
        if os.path.exists("measurements.json"):
            with open("measurements.json", "r") as file:
                measurements = json.load(file)  # Load JSON data from the file
        else:
            measurements = {}
        measurements['starting_weights_filename'] = env.get(
            'CM_ML_MODEL_STARTING_WEIGHTS_FILENAME', env.get(
                'CM_ML_MODEL_FILE', measurements.get(
                    'starting_weights_filename', '')))
        measurements['retraining'] = env.get(
            'CM_ML_MODEL_RETRAINING', measurements.get(
                'retraining', 'no'))
        measurements['input_data_types'] = env.get(
            'CM_ML_MODEL_INPUTS_DATA_TYPE', measurements.get(
                'input_data_types', 'fp32'))
        measurements['weight_data_types'] = env.get(
            'CM_ML_MODEL_WEIGHTS_DATA_TYPE', measurements.get(
                'weight_data_types', 'fp32'))
        measurements['weight_transformations'] = env.get(
            'CM_ML_MODEL_WEIGHT_TRANSFORMATIONS', measurements.get(
                'weight_transformations', 'none'))

        os.chdir(output_dir)

        if not os.path.exists("mlperf_log_summary.txt"):
            return {'return': 0}

        mlperf_log_summary = ''
        if os.path.isfile("mlperf_log_summary.txt"):
            with open("mlperf_log_summary.txt", "r") as fp:
                mlperf_log_summary = fp.read()

        if mlperf_log_summary != '':
            state['app_mlperf_inference_log_summary'] = {}
            for x in mlperf_log_summary.split('\n'):
                y = x.split(': ')
                if len(y) == 2:
                    state['app_mlperf_inference_log_summary'][y[0].strip().lower()
                                                              ] = y[1].strip()

        if env.get("CM_MLPERF_PRINT_SUMMARY", "").lower() not in [
                "no", "0", "false"]:
            print("\n")
            print(mlperf_log_summary)

        with open("measurements.json", "w") as fp:
            json.dump(measurements, fp, indent=2)

        cm_sut_info = {}
        cm_sut_info['system_name'] = state['CM_SUT_META']['system_name']
        cm_sut_info['implementation'] = env['CM_MLPERF_IMPLEMENTATION']
        cm_sut_info['device'] = env['CM_MLPERF_DEVICE']
        cm_sut_info['framework'] = state['CM_SUT_META']['framework']
        cm_sut_info['run_config'] = env['CM_MLPERF_INFERENCE_SUT_RUN_CONFIG']
        with open(os.path.join(result_sut_folder_path, "cm-sut-info.json"), "w") as fp:
            json.dump(cm_sut_info, fp, indent=2)

        system_meta = state['CM_SUT_META']
        with open("system_meta.json", "w") as fp:
            json.dump(system_meta, fp, indent=2)

        # map the custom model for inference result to the official model
        # if custom model name is not set, the official model name will be
        # mapped to itself
        official_model_name = model
        model_mapping = {model_full_name: official_model_name}
        with open("model_mapping.json", "w") as fp:
            json.dump(model_mapping, fp, indent=2)

        # Add to the state
        state['app_mlperf_inference_measurements'] = copy.deepcopy(
            measurements)

        if os.path.exists(env['CM_MLPERF_CONF']):
            shutil.copy(env['CM_MLPERF_CONF'], 'mlperf.conf')

        if os.path.exists(env['CM_MLPERF_USER_CONF']):
            shutil.copy(env['CM_MLPERF_USER_CONF'], 'user.conf')

        result, valid, power_result = mlperf_utils.get_result_from_log(
            env['CM_MLPERF_LAST_RELEASE'], model, scenario, output_dir, mode, env.get('CM_MLPERF_INFERENCE_SOURCE_VERSION'))
        power = None
        power_efficiency = None
        if power_result:
            power_result_split = power_result.split(",")
            if len(power_result_split) == 2:  # power and power efficiency
                power = power_result_split[0]
                power_efficiency = power_result_split[1]

        state['cm-mlperf-inference-results'][state['CM_SUT_CONFIG_NAME']
                                             ][model][scenario][mode] = result
        state['cm-mlperf-inference-results'][state['CM_SUT_CONFIG_NAME']
                                             ][model][scenario][mode + '_valid'] = valid.get(mode, False)

        state['cm-mlperf-inference-results-last'][mode] = result
        state['cm-mlperf-inference-results-last'][mode +
                                                  '_valid'] = valid.get(mode, False)

        if power:
            state['cm-mlperf-inference-results'][state['CM_SUT_CONFIG_NAME']
                                                 ][model][scenario]['power'] = power
            state['cm-mlperf-inference-results'][state['CM_SUT_CONFIG_NAME']
                                                 ][model][scenario]['power_valid'] = valid['power']
            state['cm-mlperf-inference-results-last']['power'] = power
            state['cm-mlperf-inference-results-last']['power_valid'] = valid['power']
        if power_efficiency:
            state['cm-mlperf-inference-results'][state['CM_SUT_CONFIG_NAME']
                                                 ][model][scenario]['power_efficiency'] = power_efficiency
            state['cm-mlperf-inference-results-last']['power_efficiency'] = power_efficiency

        # Record basic host info
        host_info = {
            "os_version": platform.platform(),
            "cpu_version": platform.processor(),
            "python_version": sys.version,
            "cm_version": cm.__version__
        }

        x = ''
        if env.get('CM_HOST_OS_FLAVOR', '') != '':
            x += env['CM_HOST_OS_FLAVOR']
        if env.get('CM_HOST_OS_VERSION', '') != '':
            x += ' ' + env['CM_HOST_OS_VERSION']
        if x != '':
            host_info['os_version_sys'] = x

        if env.get('CM_HOST_SYSTEM_NAME', '') != '':
            host_info['system_name'] = env['CM_HOST_SYSTEM_NAME']

        # Check CM automation repository
        repo_name = 'mlcommons@mlperf-automations'
        repo_hash = ''
        r = cm.access({'action': 'find', 'automation': 'repo',
                      'artifact': 'mlcommons@cm4mlops,9e97bb72b0474657'})
        if r['return'] == 0 and len(r['list']) == 1:
            repo_path = r['list'][0].path
            if os.path.isdir(repo_path):
                repo_name = os.path.basename(repo_path)

                # Check dev
                # if repo_name == 'cm4mlops': repo_name = 'mlcommons@cm4mlops'

                r = cm.access({'action': 'system',
                               'automation': 'utils',
                               'path': repo_path,
                               'cmd': 'git rev-parse HEAD'})
                if r['return'] == 0 and r['ret'] == 0:
                    repo_hash = r['stdout']

                    host_info['cm_repo_name'] = repo_name
                    host_info['cm_repo_git_hash'] = repo_hash

        with open("cm-host-info.json", "w") as fp:
            fp.write(json.dumps(host_info, indent=2) + '\n')

        # Prepare README
        if "cmd" in inp:
            cmd = "cm run script \\\n\t" + " \\\n\t".join(inp['cmd'])
            xcmd = "cm run script " + xsep + "\n\t" + \
                (" " + xsep + "\n\t").join(inp['cmd'])
        else:
            cmd = ""
            xcmd = ""

        readme_init = "*Check [CM MLPerf docs](https://docs.mlcommons.org/inference) for more details.*\n\n"

        readme_body = "## Host platform\n\n* OS version: {}\n* CPU version: {}\n* Python version: {}\n* MLCommons CM version: {}\n\n".format(platform.platform(),
                                                                                                                                             platform.processor(), sys.version, cm.__version__)

        x = repo_name
        if repo_hash != '':
            x += ' --checkout=' + str(repo_hash)

        readme_body += "## CM Run Command\n\nSee [CM installation guide](https://docs.mlcommons.org/inference/install/).\n\n" + \
            "```bash\npip install -U cmind\n\ncm rm cache -f\n\ncm pull repo {}\n\n{}\n```".format(
                x, xcmd)

        readme_body += "\n*Note that if you want to use the [latest automation recipes](https://docs.mlcommons.org/inference) for MLPerf (CM scripts),\n" + \
                       " you should simply reload {} without checkout and clean CM cache as follows:*\n\n".format(repo_name) + \
                       "```bash\ncm rm repo {}\ncm pull repo {}\ncm rm cache -f\n\n```".format(
                           repo_name, repo_name)

        extra_readme_init = ''
        extra_readme_body = ''
        if env.get('CM_MLPERF_README', '') == "yes":
            extra_readme_body += "\n## Dependent CM scripts\n\n"

            script_tags = inp['tags']
            script_adr = inp.get('adr', {})

            cm_input = {'action': 'run',
                        'automation': 'script',
                        'tags': script_tags,
                        'adr': script_adr,
                        'print_deps': True,
                        'env': env,
                        'quiet': True,
                        'silent': True,
                        'fake_run': True
                        }
            r = cm.access(cm_input)
            if r['return'] > 0:
                return r

            print_deps = r['new_state']['print_deps']
            count = 1
            for dep in print_deps:
                extra_readme_body += "\n\n" + str(count) + ".  `" + dep + "`\n"
                count = count + 1

            if state.get(
                    'mlperf-inference-implementation') and state['mlperf-inference-implementation'].get('print_deps'):

                extra_readme_body += "\n## Dependent CM scripts for the MLPerf Inference Implementation\n"

                print_deps = state['mlperf-inference-implementation']['print_deps']
                count = 1
                for dep in print_deps:
                    extra_readme_body += "\n\n" + \
                        str(count) + ". `" + dep + "`\n"
                    count = count + 1

        readme = readme_init + readme_body
        extra_readme = extra_readme_init + extra_readme_body

        with open("README.md", "w") as fp:
            fp.write(readme)
        if extra_readme:
            with open("README-extra.md", "w") as fp:
                fp.write(extra_readme)

    elif mode == "compliance":

        test = env.get("CM_MLPERF_LOADGEN_COMPLIANCE_TEST", "TEST01")

        RESULT_DIR = os.path.split(output_dir)[0]
        COMPLIANCE_DIR = output_dir
        OUTPUT_DIR = os.path.dirname(COMPLIANCE_DIR)

        SCRIPT_PATH = os.path.join(
            env['CM_MLPERF_INFERENCE_SOURCE'],
            "compliance",
            "nvidia",
            test,
            "run_verification.py")
        if test == "TEST06":
            cmd = f"{env['CM_PYTHON_BIN_WITH_PATH']}  {SCRIPT_PATH}  -c  {COMPLIANCE_DIR}  -o  {OUTPUT_DIR} --scenario {scenario} --dtype int32"
        else:
            cmd = f"{env['CM_PYTHON_BIN_WITH_PATH']}  {SCRIPT_PATH}  -r {RESULT_DIR} -c  {COMPLIANCE_DIR}  -o  {OUTPUT_DIR}"

        print(cmd)
        os.system(cmd)

        if test == "TEST01":

            run_script_input = i['run_script_input']
            automation = i['automation']

            SCRIPT_PATH = os.path.join(env['CM_MLPERF_INFERENCE_SOURCE'], "compliance", "nvidia", test,
                                       "create_accuracy_baseline.sh")
            TEST01_DIR = os.path.join(OUTPUT_DIR, "TEST01")
            OUTPUT_DIR = os.path.join(OUTPUT_DIR, "TEST01", "accuracy")
            if not os.path.exists(OUTPUT_DIR):
                os.makedirs(OUTPUT_DIR)

            ACCURACY_DIR = os.path.join(RESULT_DIR, "accuracy")
            if not os.path.exists(ACCURACY_DIR):
                print("Accuracy run not yet completed")
                return {
                    'return': 1, 'error': 'TEST01 needs accuracy run to be completed first'}

            cmd = "cd " + TEST01_DIR + " &&  bash " + SCRIPT_PATH + " " + os.path.join(ACCURACY_DIR, "mlperf_log_accuracy.json") + " " + \
                os.path.join(COMPLIANCE_DIR, "mlperf_log_accuracy.json")
            env['CMD'] = cmd
            print(cmd)
            r = automation.run_native_script(
                {'run_script_input': run_script_input, 'env': env, 'script_name': 'verify_accuracy'})
            if r['return'] > 0:
                return r

            verify_accuracy_file = os.path.join(
                TEST01_DIR, "verify_accuracy.txt")
            with open(verify_accuracy_file, 'r') as file:
                data = file.read().replace('\n', '\t')

            if 'TEST PASS' not in data:
                print("\nDeterministic TEST01 failed... Trying with non-determinism.\n")
            # #Normal test failed, trying the check with non-determinism

                baseline_accuracy_file = os.path.join(
                    TEST01_DIR, "mlperf_log_accuracy_baseline.json")
                CMD = "cd " + ACCURACY_DIR + " && " + env['CM_PYTHON_BIN_WITH_PATH'] + ' ' + accuracy_filepath + accuracy_log_file_option_name + \
                    baseline_accuracy_file + ' ' + dataset_args + \
                    datatype_option + out_baseline_accuracy_string

                env['CMD'] = CMD
                r = automation.run_native_script(
                    {'run_script_input': run_script_input, 'env': env, 'script_name': 'verify_accuracy'})
                if r['return'] > 0:
                    return r

                if os.stat(baseline_accuracy_file).st_size == 0:
                    return {'return': 1,
                            'error': f"{baseline_accuracy_file} is empty"}

                CMD = "cd " + ACCURACY_DIR + " &&  " + env['CM_PYTHON_BIN_WITH_PATH'] + ' ' + accuracy_filepath + accuracy_log_file_option_name + \
                    os.path.join(TEST01_DIR, "mlperf_log_accuracy.json") + \
                    dataset_args + datatype_option + out_compliance_accuracy_string

                env['CMD'] = CMD
                r = automation.run_native_script(
                    {'run_script_input': run_script_input, 'env': env, 'script_name': 'verify_accuracy'})
                if r['return'] > 0:
                    return r
        import submission_checker as checker
        is_valid = checker.check_compliance_perf_dir(
            COMPLIANCE_DIR) if test != "TEST06" else True
        state['cm-mlperf-inference-results'][state['CM_SUT_CONFIG_NAME']
                                             ][model][scenario][test] = "passed" if is_valid else "failed"

    # portion of the code where the avg utilisation and system informations are extracted
    # NOTE: The section is under development and print statements are added
    # for further debugging
    if env.get('CM_PROFILE_NVIDIA_POWER', '') == "on":
        import pandas as pd
        system_utilisation_info_dump = {}
        logs_dir = output_dir
        # logs_dir = env.get('CM_LOGS_DIR', env['CM_RUN_DIR'])
        sys_utilisation_log = pd.read_csv(
            os.path.join(
                logs_dir,
                'sys_utilisation_info.txt'),
            dtype={
                'cpu_utilisation': float,
                'used_memory_gb': float})
        with open(os.path.join(logs_dir, 'mlperf_log_detail.txt'), 'r') as file:
            log_txt = file.read()
            # patterns for matching the power_begin and power_end in mlperf log
            pattern_begin = r'\"key\"\:\s\"power_begin\"\,\s\"value\"\:\s\"(.*?)\"'
            pattern_end = r'\"key\"\:\s\"power_end\"\,\s\"value\"\:\s\"(.*?)\"'
            # match the patterns with the text present in the log details file
            match_begin = re.findall(pattern_begin, log_txt)[0]
            match_end = re.findall(pattern_end, log_txt)[0]
            power_begin_time = pd.Timestamp(datetime.strptime(
                match_begin, '%m-%d-%Y %H:%M:%S.%f')).replace(tzinfo=timezone.utc)
            power_end_time = pd.Timestamp(datetime.strptime(
                match_end, '%m-%d-%Y %H:%M:%S.%f')).replace(tzinfo=timezone.utc)
        # converts timestamp key value to datetime objects
        sys_utilisation_log['timestamp'] = pd.to_datetime(
            sys_utilisation_log['timestamp'])
        '''
        for i in range(len(sys_utilisation_log['timestamp'])):
            print(f"{sys_utilisation_log['timestamp'][i]} {power_begin_time}")
            print(sys_utilisation_log['timestamp'][i]>=power_begin_time)
        '''
        # print(f"{sys_utilisation_log['timestamp'][0]} {power_begin_time}")
        # print(sys_utilisation_log['timestamp'][0]>=power_begin_time)
        filtered_log = sys_utilisation_log[(sys_utilisation_log['timestamp'] >= power_begin_time) &
                                           (sys_utilisation_log['timestamp'] <= power_end_time)]
        # print(filtered_log)
        # Calculate average of cpu_utilisation and used_memory_gb
        system_utilisation_info_dump["avg_cpu_utilisation"] = filtered_log['cpu_utilisation'].mean(
        )
        system_utilisation_info_dump["avg_used_memory_gb"] = filtered_log['used_memory_gb'].mean(
        )
        print("\nSystem utilisation info for the current run:")
        print(system_utilisation_info_dump)
        print("\n")

    if state.get(
            'mlperf-inference-implementation') and state['mlperf-inference-implementation'].get('version_info'):
        env['CM_MLPERF_RUN_JSON_VERSION_INFO_FILE'] = os.path.join(
            output_dir, "cm-version-info.json")
        env['CM_MLPERF_RUN_DEPS_GRAPH'] = os.path.join(
            output_dir, "cm-deps.png")
        env['CM_MLPERF_RUN_DEPS_MERMAID'] = os.path.join(
            output_dir, "cm-deps.mmd")
        with open(os.path.join(output_dir, "cm-version-info.json"), "w") as f:
            f.write(
                json.dumps(
                    state['mlperf-inference-implementation']['version_info'],
                    indent=2))

    if env.get('CM_DUMP_SYSTEM_INFO', True):
        dump_script_output(
            "detect,os",
            env,
            state,
            'new_env',
            os.path.join(
                output_dir,
                "os_info.json"))
        dump_script_output(
            "detect,cpu",
            env,
            state,
            'new_env',
            os.path.join(
                output_dir,
                "cpu_info.json"))
        env['CM_DUMP_RAW_PIP_FREEZE_FILE_PATH'] = os.path.join(
            env['CM_MLPERF_OUTPUT_DIR'], "pip_freeze.raw")
        dump_script_output(
            "dump,pip,freeze",
            env,
            state,
            'new_state',
            os.path.join(
                output_dir,
                "pip_freeze.json"))

    return {'return': 0}


def dump_script_output(script_tags, env, state, output_key, dump_file):

    cm_input = {'action': 'run',
                'automation': 'script',
                'tags': script_tags,
                'env': env,
                'state': state,
                'quiet': True,
                'silent': True,
                }
    r = cm.access(cm_input)
    if r['return'] > 0:
        return r
    with open(dump_file, "w") as f:
        f.write(json.dumps(r[output_key], indent=2))

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):
    os_info = i['os_info']

    env = i['env']
    CPPFLAGS = env.get('+ CPPFLAGS', [])
    env['CM_C_COMPILER_FLAGS'] = " ".join(env.get('+ CFLAGS', []) + CPPFLAGS)
    env['CM_CXX_COMPILER_FLAGS'] = " ".join(
        env.get('+ CXXFLAGS', []) + CPPFLAGS)
    env['CM_F_COMPILER_FLAGS'] = " ".join(env.get('+ FFLAGS', []))

    CPATH = env.get('+CPATH', [])
    env['CM_C_INCLUDE_PATH'] = " -I".join([" "] +
                                          env.get('+C_INCLUDE_PATH', []) +
                                          CPATH)
    env['CM_CPLUS_INCLUDE_PATH'] = " -I".join(
        [" "] + env.get('+CPLUS_INCLUDE_PATH', []) + CPATH)
    env['CM_F_INCLUDE_PATH'] = " -I".join([" "] +
                                          env.get('+F_INCLUDE_PATH', []) +
                                          CPATH)

    # If windows, need to extend it more ...
    if os_info['platform'] == 'windows' and env.get(
            'CM_COMPILER_FAMILY', '') != 'LLVM':
        print("WARNING: compile-program script should be extended to support flags for non-LLVM compilers on Windows")
        return {'return': 0}

    LDFLAGS = env.get('+ LDFLAGS', [])

    env['CM_C_LINKER_FLAGS'] = " ".join(env.get('+ LDCFLAGS', []) + LDFLAGS)
    env['CM_CXX_LINKER_FLAGS'] = " ".join(
        env.get('+ LDCXXFLAGS', []) + LDFLAGS)
    env['CM_F_LINKER_FLAGS'] = " ".join(env.get('+ LDFFLAGS', []) + LDFLAGS)

    if env.get('CM_LINKER_LANG', 'C') == "C":
        env['CM_LINKER_BIN'] = env['CM_C_COMPILER_BIN']
        env['CM_LINKER_WITH_PATH'] = env['CM_C_COMPILER_WITH_PATH']
        env['CM_LINKER_COMPILE_FLAGS'] = env['CM_C_COMPILER_FLAGS']
        env['CM_LINKER_FLAGS'] = env['CM_C_LINKER_FLAGS']

    elif env.get('CM_LINKER_LANG', 'C') == "CXX":
        env['CM_LINKER_BIN'] = env['CM_CXX_COMPILER_BIN']
        env['CM_LINKER_WITH_PATH'] = env['CM_CXX_COMPILER_WITH_PATH']
        env['CM_LINKER_COMPILE_FLAGS'] = env['CM_CXX_COMPILER_FLAGS']
        env['CM_LINKER_FLAGS'] = env['CM_CXX_LINKER_FLAGS']

    elif env.get('CM_LINKER_LANG', 'C') == "F":
        env['CM_LINKER_BIN'] = env['CM_F_COMPILER_BIN']
        env['CM_LINKER_WITH_PATH'] = env['CM_F_COMPILER_WITH_PATH']
        env['CM_LINKER_COMPILE_FLAGS'] = env['CM_F_COMPILER_FLAGS']
        env['CM_LINKER_FLAGS'] = env['CM_F_LINKER_FLAGS']

    env['CM_LD_LIBRARY_PATH'] = " -L".join([" "] +
                                           env.get('+LD_LIBRARY_PATH', []))
    env['CM_SOURCE_FOLDER_PATH'] = env['CM_SOURCE_FOLDER_PATH'] if 'CM_SOURCE_FOLDER_PATH' in env else env[
        'CM_TMP_CURRENT_SCRIPT_PATH'] if 'CM_TMP_CURRENT_SCRIPT_PATH' in env else ''

    return {'return': 0}


def postprocess(i):

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']
    env = i['env']

    if env.get('CM_TMP_MLPERF_INFERENCE_LOADGEN_INSTALL_FROM_PIP', '') == 'yes':
        i['run_script_input']['script_name'] = "donotrun"

    return {'return': 0}


def postprocess(i):

    os_info = i['os_info']
    env = i['env']

    if env.get('CM_TMP_MLPERF_INFERENCE_LOADGEN_INSTALL_FROM_PIP', '') == 'yes':
        return {'return': 0}

    for key in ['+PYTHONPATH', '+C_INCLUDE_PATH', '+CPLUS_INCLUDE_PATH',
                '+LD_LIBRARY_PATH', '+DYLD_FALLBACK_LIBRARY_PATH']:
        #        20221024: we save and restore env in the main script and can clean env here for determinism
        #        if key not in env:
        env[key] = []

    # On Windows installs directly into Python distro for simplicity
#    if os_info['platform'] != 'windows':

    cur_path = os.getcwd()
    install_path = os.path.join(cur_path, 'install')

    env['CM_MLPERF_INFERENCE_LOADGEN_INSTALL_PATH'] = install_path

    build_path = os.path.join(cur_path, 'build')
    if os.path.exists(build_path):
        env['CM_MLPERF_INFERENCE_LOADGEN_BUILD_PATH'] = build_path

    include_path = os.path.join(install_path, 'include')
    lib_path = os.path.join(install_path, 'lib')
    python_path = os.path.join(install_path, 'python')

    env['+C_INCLUDE_PATH'].append(include_path)
    env['+CPLUS_INCLUDE_PATH'].append(include_path)
    env['CM_MLPERF_INFERENCE_LOADGEN_INCLUDE_PATH'] = include_path

    env['+LD_LIBRARY_PATH'].append(lib_path)
    env['+DYLD_FALLBACK_LIBRARY_PATH'].append(lib_path)
    env['CM_MLPERF_INFERENCE_LOADGEN_LIBRARY_PATH'] = lib_path

    env['+PYTHONPATH'].append(python_path)
    env['CM_MLPERF_INFERENCE_LOADGEN_PYTHON_PATH'] = python_path

    return {'return': 0}



# Developer(s): Grigori Fursin

import cmind
import os
import misc

import streamlit.components.v1 as components

import streamlit as st

announcement = 'Under development - please get in touch via [Discord](https://discord.gg/JjWNWXKxwT) for more details ...'


def main():
    params = misc.get_params(st)

    # Set title
    st.title('How to run benchmarks')

    st.markdown(announcement)

    return page(st, params)


def page(st, params, action=''):

    end_html = ''

    # Announcement
#    st.markdown('----')

    url_script = misc.make_url('', key='', action='scripts', md=False)

    # Some info
    x = '''
         <i>
         <small>
         This interface will help you generate a command line or Python API
         to run modular benchmarks composed from
         <a href="{}">automation recipes (CM scripts)</a>.
         Note that this is a <a href="https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md">collaborative engineering effort</a>
         to make sure that they work across all possible versions and configurations of models, data sets, software and hardware
         - please report encountered issues and provide feedback
         <a href="https://github.com/mlcommons/ck/issues">here</a>
         and get in touch via <a href="https://discord.gg/JjWNWXKxwT">Discord</a>!
         </small>
         </i>
          <br>
          <br>
        '''.format(url_script)

    st.write(x, unsafe_allow_html=True)

#    st.markdown(announcement)

    ##########################################################################
    # Select target hardware
    compute_uid = ''
    x = params.get('compute_uid', [''])
    if len(x) > 0 and x[0] != '':
        compute_uid = x[0].strip()

    ii = {'action': 'load_cfg',
          'automation': 'utils',
          'tags': 'benchmark,compute',
          'skip_files': False}

    if compute_uid != '':
        ii['prune'] = {'uid': compute_uid}

    r = cmind.access(ii)
    if r['return'] > 0:
        return r

    r = misc.make_selection(
        st,
        r['selection'],
        'compute',
        'target hardware',
        compute_uid)
    if r['return'] > 0:
        return r

    compute_meta = r['meta']
#    st.markdown(compute_meta)

    ##########################################################################
    # Select benchmark
    bench_uid = ''
    x = params.get('bench_uid', [''])
    if len(x) > 0 and x[0] != '':
        bench_uid = x[0].strip()

    ii = {'action': 'load_cfg',
          'automation': 'utils',
          'tags': 'benchmark,list',
          'skip_files': False}

    if bench_uid != '':
        ii['prune'] = {'uid': bench_uid}

    r = cmind.access(ii)
    if r['return'] > 0:
        return r

    # Prune by supported compute
    selection = r['selection']
    pruned_selection = []

    if len(compute_meta) == 0 or compute_meta.get('tags', '') == '':
        pruned_selection = selection
    else:
        xtags = set(compute_meta['tags'].split(','))

#        st.markdown(str(xtags))

        for s in selection:
            add = True

            supported_compute = s.get('supported_compute', [])
            if len(supported_compute) > 0:
                add = False

                for c in supported_compute:
                    cc = set(c.split(','))
                    if cc.issubset(xtags):
                        add = True
                        break

            if add:
                pruned_selection.append(s)

    # Make default selection of MLPerf inference
    force_bench_index = 0
    if bench_uid == '':
        j = 0
        for q in sorted(pruned_selection, key=lambda v: v['name']):
            j += 1
            if q['uid'] == '39877bb63fb54725':
                force_bench_index = j

    r = misc.make_selection(
        st,
        pruned_selection,
        'benchmark',
        'benchmark',
        bench_uid,
        force_index=force_bench_index)
    if r['return'] > 0:
        return r

    bench_meta = r['meta']
#    st.markdown(bench_meta)

    if len(bench_meta) > 0:
        #######################################################################
        # Check common CM interface

        #        st.markdown('---')

        urls = bench_meta.get('urls', [])

        script_path = ''
        script_name = bench_meta.get('script_name', '')
        script_meta = {}
        script_obj = None
        script_url = ''
        if script_name != '':
            ii = {'action': 'find',
                  'automation': 'script',
                  'artifact': script_name}
            r = cmind.access(ii)
            if r['return'] > 0:
                return r

            lst = r['list']

            if len(lst) > 0:

                script_obj = lst[0]

                script_meta = script_obj.meta
                script_path = script_obj.path
                script_repo_meta = script_obj.repo_meta

                script_alias = script_meta['alias']

                repo_meta = script_obj.repo_meta

                url = repo_meta.get('url', '')
                if url == '' and repo_meta.get('git', False):
                    url = 'https://github.com/' + \
                        repo_meta['alias'].replace('@', '/')

                if url != '':
                    # Recreate GitHub path
                    if not url.endswith('/'):
                        url = url + '/'

                    url += 'tree/master/'

                    if repo_meta.get('prefix', '') != '':
                        url += repo_meta['prefix']

                    if not url.endswith('/'):
                        url = url + '/'

                    url += 'script/' + script_alias

                    script_url = url

                if not bench_meta.get('skip_extra_urls', False):
                    url_script = misc.make_url(
                        script_name, key='name', action='scripts', md=False)
                    url_script += '&gui=true'

                    urls.append({'name': 'Universal CM GUI to run this benchmark',
                                 'url': url_script})

                    # Check if extra README
                    script_path_readme_extra = os.path.join(
                        script_path, 'README-extra.md')

                    if os.path.isfile(script_path_readme_extra):
                        # Check README.extra.md
                        url_readme_extra = url + '/README-extra.md'

                        urls.append({'name': 'Notes about how to run this benchmark from the command line',
                                     'url': url_readme_extra})

        # Check URLS
        if len(urls) > 0:
            x = '\n'
            for u in urls:
                name = u['name']
                url = u['url']

                x += '* [{}]({})\n'.format(name, url)
            x += '\n'

            st.markdown(x)

        #######################################################################
        # Check if has customization
        extra = {}
        skip = False

        script_tags = script_meta.get('tags_help', '')
        if script_tags == '':
            script_tags = ','.join(script_meta.get('tags', []))

        if script_obj is not None:
            ii = {'st': st,
                  'params': params,
                  'meta': script_obj.meta,
                  'misc_module': misc,
                  'compute_meta': compute_meta,
                  'bench_meta': bench_meta,
                  'script_path': script_path,
                  'script_tags': script_tags,
                  'script_url': script_url}

            import sys
            import importlib

            full_module_path = os.path.join(script_obj.path, 'customize.py')

            tmp_module = None
            try:
                found_automation_spec = importlib.util.spec_from_file_location(
                    'customize', full_module_path)
                if found_automation_spec is not None:
                    tmp_module = importlib.util.module_from_spec(
                        found_automation_spec)
                    found_automation_spec.loader.exec_module(tmp_module)
#               tmp_module=importlib.import_module('customize')
            except Exception as e:
                st.markdown('WARNING: {}'.format(e))
                pass

            if tmp_module is not None:
                if hasattr(tmp_module, 'gui'):
                    try:
                        func = getattr(tmp_module, 'gui')
                    except Exception as e:
                        return {'return': 1, 'error': format(e)}

                    r = func(ii)
                    if r['return'] > 0:
                        return r

                    extra = r.get('extra', {})
                    skip = r.get('skip', False)

        #######################################################################
        # Show official GUI
        if script_path != '' and not skip:
            import script

            ii = {'st': st,
                  'params': params,
                  'script_path': script_path,
                  'script_alias': script_alias,
                  'script_tags': script_tags,
                  'script_meta': script_meta,
                  'script_repo_meta': script_repo_meta,
                  'skip_bottom': True,
                  'extra': extra}

            rr = script.page(ii)
            if rr['return'] > 0:
                return rr

            end_html += '\n' + rr.get('end_html', '')

        #######################################################################
        self_url = misc.make_url(
            bench_meta['uid'],
            action='howtorun',
            key='bench_uid',
            md=False)

        if len(compute_meta) > 0:
            self_url += '&compute_uid=' + compute_meta['uid']

        end_html = '<center><small><i><a href="{}">Self link</a></i></small></center>'.format(
            self_url)

    return {'return': 0, 'end_html': end_html}
